@@ -1,6 +1,6 @@
 #include "survPresmooth.h"
 
-void simpson(double *integrand, int *lintegrand, double *step, double *integral){
+void simpson(double *integrand, int *lintegrand, double *integral){
 	int i;
 	div_t resdiv;
 
@@ -8,11 +8,11 @@ void simpson(double *integrand, int *lintegrand, double *step, double *integral)
 	for (i = 1; i < (*lintegrand - 1); i++){
 		resdiv = div(i, 2);
     	if(resdiv.rem == 0)
-			*integral += 2 * integrand[i];
+			*integral += 2.0 * integrand[i];
 		else 
-			*integral += 4 * integrand[i];
+			*integral += 4.0 * integrand[i];
 	}
 	*integral += integrand[i];
-	*integral *= *step / 3;
+	*integral /= 3.0;
 }
 
